%%%%%%%%%%%%%%%%%%%%%% cp10_2.m %%%%%%%%%%%%%%%%%%%%%
%       Feedback Control Problems with MATLAB
%           and the Control System Toolbox
%      D. K. Frederick and J. H. Chow, May 1999
%             last revision: 6/15/99 HS
%----------------------------------------------------
%	     Comprehensive Problem CP10.2
%    does observer-controller design for satellite
%----------------------------------------------------
clear,clf
disp('CP10_2: observer-controller design for ')
disp('...... reduced model of satellite')
% load sat.mat if exists, otherwise quit this M-file
if chkmat('sat') == 1, 
    load sat.mat
  else
     return
end
%---- plant matrices, 3 states, 1 input, 2 outputs
Ap = A;
Bp = B;
disp('******>');pause
Cp = C;
Dp = D;
Gp = ss(Ap,Bp,Cp,Dp)

disp('******>');pause
% 
disp('test for controllability')
Co = ctrb(Ap,Bp)
n_Co = rank(Co)
disp('3rd-order model is NOT controllable')
disp('******>');pause
Ob = obsv(Ap,Cp)
n_Ob = rank(Ob)
disp('3rd-order model is observable')
disp('******>');pause
disp('Because 3rd-order model is NOT controllable')
disp('  we can eliminate 3rd state ==> wheel speed')
disp('  and work with the 2nd-order model having:')
disp('    theta & theta-dot as states')
disp('    input is torque to reaction wheel')
disp('    output is theta')
disp('Doing this gives the following state-space matrices:')

Ap1 = Ap(1:2,1:2)
Bp1 = Bp(1:2,:)
disp('******>');pause
Cp1 = Cp(1,1:2)
Dp1 = Dp(1,:)

disp('******>');pause
disp('reasonable values for CL eigenvalues are')
disp('     s = -0.1 and s= -0.05 which can be')
disp('   entered as  [-0.1; -0.05]')
pp = input('enter 2 desired CL eigenvalues as column ==> ')
Rpp_str = vec2str(real(pp)); % display as real & imag
Ipp_str = vec2str(imag(pp)); % .....on plot
disp('controller gains are:')
F = place(Ap1,Bp1,pp)
disp('******>');pause
disp('reasonable values for observer poles are')
disp('     s = -0.2 and s= -0.3')
disp('   entered as  [-0.2; -0.3]')
qq = input('enter 2 desired observer poles as column ==> ')
Rqq_str = vec2str(real(qq));  % display as real & imag
Iqq_str = vec2str(imag(qq));  % .....on plot
disp('observer gains are:')
L = place(Ap1',Cp1',qq)'
disp('******>');pause
disp('SS model of 2nd-order observer-controller is:')

Aoc = Ap1-Bp1*F-L*Cp1	% observer-controller (2x2)
Boc = L			%     (2x1)
disp('******>');pause
Coc = -F		%     (1x2)
Doc = 0			%     (1x1)
Goc = ss(Aoc,Boc,Coc,Doc);

disp('******>');pause
disp('eigenvalues of 2nd-order observer-controller are:')
pOC=eig(Aoc)		% obsvr-ctrlr poles
disp('******>');pause
disp('zeros of 2nd-order observer-controller are:')
zOC=tzero(Goc)	% obsv-ctrlr zeros
%--- series connection of obsvr-ctrlr & full plant
disp('connect observer-controller in series ')
disp('......with 3rd-order plant')

Gocp = Goc*Gp;
[Aocp,Bocp,Cocp,Docp] = ssdata(Gocp); % 1 input,2 outputs

disp('Make unity-fdbk connection using theta')
disp('.... with + sign at summing junction')
outputs = 1;
inputs = 1;

Gcl = feedback(Gocp,1,inputs,outputs);
[Acl,Bcl,Ccl,Dcl] = ssdata(Gcl);

%---- unit step in desired pointing angle at t = 10 s
t=[0:0.5:200]';
theta_ref = -ones(size(t));
for i = find(t<10)',
   theta_ref(i) = 0;
end
[y,x] = lsim(Gcl,theta_ref,t);
theta = y(:,1);
Omega = y(:,2);
subplot(2,1,1)
plot(t,theta,t,-theta_ref,'--');grid
title('CP10.2: satellite; obsv-ctrlr with theta fedback')
xlabel('time (secs)')
ylabel('pointing angle (deg)')
text('Units','norm','Pos',[0.5 0.40],...
			'Str','Controller poles')
text('Units','norm','Pos',[0.5 0.25],...
		'Str',['real parts = ' Rpp_str])
text('Units','norm','Pos',[0.5 0.10],...
		'Str',['imag parts = ' Ipp_str])
subplot(2,1,2)
plot(t,Omega);grid
ylabel('wheel speed (rpm)')
xlabel('time (secs)')
text('Units','norm','Pos',[0.5 0.40],...
			'Str','Observer poles')
text('Units','norm','Pos',[0.5 0.25],...
		'Str',['real parts = ' Rqq_str])
text('Units','norm','Pos',[0.5 0.10],...
		'Str',['imag parts = ' Iqq_str])
keyboard
%---- calc time-domain response specs for pointing angle
[Mo,tp,tr,ts,ess] = tstats(t,theta)
disp('peak wheel-speed magnitude in rpm is')
max_speed = max(abs(Omega));
disp(max_speed)
keyboard
disp('end of Comprehensive Problem 10.2')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
