%%%%%%%%%%%%%%%%%%%%%% ch4ex.m %%%%%%%%%%%%%%%%%%%%%%
%       Feedback Control Problems with MATLAB
%           and the Control System Toolbox
%      D. K. Frederick and J. H. Chow, Dec. 98
%----------------------------------------------------
%  This MATLAB file runs the 5 examples that appear 
%  in Chapter 4 (State-Space Models) in the BookWare 
%  Companion Feedback Control Problems Using MATLAB
%          by D. K. Frederick and J. H. Chow 
%       *************************************
%       *   requires the special functions: *
%       *   cpole2t.m, rpole2t.m 
%       *************************************
%----------------------------------------------------
clear
%  menu to select Example to be run
example = -1;      % force menu to appear
while (example < 1) | (example > 9),
  disp(' 1 <= Example 4.1: Building a state-space model')
  disp(' 2 <= Example 4.2: Series, parallel and feddback connections')
  disp(' 3 <= Example 4.3: Eigenvalues and transmission zeros')
  disp(' 4 <= Example 4.4: Impulse, step and general response')
  disp(' 5 <= Example 4.5: Modal transformation')
  example = input('Select an example, or enter 0 to return to chapter menu ==> ');

if example == 0, disp('Completed Chapter 4 examples'), return, end

if example == 1,
  disp('Example 4.1: Building a state-space model')
  %   
  disp(' ')
  disp('Enter A, B, C, D matrices')
  A = [0 1 0; -4 -1 1; 0 0 -20]
  B = [0; 0; 20], C = [1 0 0], D =0
  disp('*****>');pause
  disp('Set up model in SS form')
  Gss = ss(A,B,C,D)
  disp('*****>');pause
  % convert to transfer function form
  Gtf = tf(Gss)
  disp('*****>');pause
  % convert back to state-space form
  Gss1 = ss(Gtf)
  disp('*****>');pause
  % convert to ZP form
  Gzpk = zpk(Gss)
  disp('*****>');pause
  % convert back to SS form
  Gss2 = ss(Gzpk)
  disp('*****>');pause
  clear numG denG A B C D Gss Gtf Gss1 Gzpk Gss2
  % student - do a step response of all 3 
  disp('end of Example 4.1'); disp('  ')
  example =-1;     % force menu to appear
end     % of example 4.1 so return to menu
%-------------------------------------

if example == 2,
  disp('Example 4.2: Series, parallel and feedback connections')
  disp(' ')
  disp('State-space matrices of G1(s)')
  A1 = [0 1; -3 -5], B1 = [0; 1]      %  G1 state matrices
  C1 = [1 2], D1 = 0  
  disp('*****>');pause
  disp('G1(s) as SS object')
  G1 = ss(A1,B1,C1,D1)
  disp('*****>');pause
  disp('State-space matrices of G2(s)')
  A2 = [-3 1; 0 -4], B2 = [0; 4]      %  G2 state matrices
  C2 = [3 0], D2 = 2                     
  disp('*****>');pause
  disp('G2(s) as SS object')
  G2 = ss(A2,B2,C2,D2)
  disp('*****>');pause
  disp('Series connection Ts(s)')
  Ts = G1*G2 
  disp('*****>');pause
  disp('Parallel connection Tp(s)')
  Tp = G1+G2
  disp('*****>');pause
  disp('Feedback connection Tf(s)')
  Tf = feedback(G1,G2)
  disp('*****>');pause
  clear A1 B1 C1 D1 A2 B2 C2 D2 Ts Tp Tf
  disp('end of Example 4.2'); disp('  ')
  example =-1;     % force menu to appear
end     % of Example 4.2 so return to menu
%-------------------------------------
if example == 3,
  disp('Example 4.3: Eigenvalues and transmission zeros')
  disp(' ')
  disp('System A, B, C, D matrices')
  A = [0 1 0; -5 -1 1; 0 0 -10]
  B = [0; 0; 10], C = [1 0 1], D =0
  disp('*****>');pause
  disp('System as SS object')
  G = ss(A,B,C,D)
  disp('*****>');pause
  disp('Zeros, poles, and gain of System')
  [zG,pG,kG] = zpkdata(G,'v')
  disp('*****>');pause
  disp('Eigevnalues of A')
  eigA = eig(A) 
  disp('*****>');pause
  disp('System transmission zeros')
  zero = tzero(G)
  disp('*****>');pause
  pzmap(G), axis([-12 0 -3 3])
  set(findobj('marker','x'),'markersize',12)
  set(findobj('marker','x'),'linewidth',1.5)
  set(findobj('marker','o'),'markersize',9)
  set(findobj('marker','o'),'linewidth',1.5)
  grid
  % student - fix up the axis of the plot, & marker size
  clear A B C D zG pG kG eigA G  
  disp('end of Example 4.3'); disp('  ')
  example =-1;     % force menu to appear
end     % of Example 4.3 so return to menu
%-------------------------------------

if example == 4,
  clf
  disp('Example 4.4: Impulse, step and general response')
  disp(' ')
  disp('System A, B, C, D matrices')
  A = [-2 -2.5 -0.5; 1 0 0; 0 1 0] 
  B = [1; 0; 0]  
  C = [0 1.5 1] 
  D = 0
  disp('*****>');pause
  disp('System as SS object')
  G = ss(A,B,C,D)
  disp('*****>');pause
  disp('Simulate impulse response')
  t = [0:0.1:20]';
  impulse(G,t)           % generate impulse response
  %title('Ex 4.4: Impulse response')
  grid
  keyboard
  disp('Simulate step response')
  step(G,t)              % generate step response
  %title('Ex 4.4: Step response')
  grid
  keyboard
  disp('Enter initial condition')
  xo = [1; 0; 2]                 % initial condition
  u(1,1:20) = 2*ones(1,20);      % input for 0<t<2
  u(1,21:201) = 0.5*ones(1,181); % input for t>2
  disp('Simulate response due to input and initial condition')
  [y,t,x] = lsim(G,u,t,xo);  % response due to input and 
			         %   initial condition
  plot(t,x(:,1),'-',t,x(:,2),'--',t,x(:,3),'-.')
  %title('Ex 4.4: State response, x(t)')
  xlabel('Time (secs)')
  ylabel('Amplitude')
  grid
  text(6,0.3,'x_1(t)')
  text(6,-0.5,'x_2(t)')
  text(8,1.8,'x_3(t)')
  keyboard
  clear A B C D G t u xo y x
  disp('end of Example 4.4'); disp('  ')
  example =-1;     % force menu to appear
end     % of Example 4.4 so return to menu
%-------------------------------------

if example == 5,
  disp('Example 5.5: Modal transformation')
  disp('System A, B, C, D matrices')
  A = [0 1 0; 0 0 1; -0.5 -2.5 -2]
  B = [0; 0; 1]  
  C = [1 -1.5 0] 
  D = 0
  disp('*****>');pause
  disp('System as SS object')
  G = ss(A,B,C,D)
  disp('*****>');pause
  disp('Compute eigenvalues and eigenvectors')
  [M Ad] = eig(A)                   
  disp('*****>');pause
  Bd = inv(M)*B
  Cd = C*M
  disp('*****>');pause
  disp('Perform real modal form transformation')
  Gm = canon(G,'modal')
  disp('*******>');pause
  clear A B C D T G Gm 
  disp('end of Example 4.5'); disp('  ')
  example =-1;     % force menu to appear
end     % of Example 4.5 so return to menu
%
%-----------
end             % of while
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
