%%%%%%%%%%%%%%%%%%%%%% cp4_2.m %%%%%%%%%%%%%%%%%%%%%%
%       Feedback Control Problems with MATLAB
%           and the Control System Toolbox
%      D. K. Frederick and J. H. Chow, May 1999
%             last revision: 6/2/99 HS
%----------------------------------------------------
%             Comprehensive Problem CP4.2:
%    satellite with only theta & Omega as outputs
%           but with only theta feedback
%         Series lead with unity feedback.
%----------------------------------------------------
disp('CP4.2: Satellite with only theta as output')
clear;clf
% load sat.mat if exists, otherwise quit this M-file
if chkmat('sat') == 1, 
    load sat.mat
  else
     return
end
%
A
B
disp('*******>'),pause
C
D
disp('*******>'),pause
disp('Create lead controller with these parameters')
Klead = -0.001, Zlead = -0.02, alpha = 10
Klead_str=num2str(Klead);
Zlead_str=num2str(Zlead);
alpha_str=num2str(alpha);
disp('*******>'),pause
disp('lead controller transfer function in TF form')
Gss = ss(A,B,C,D)
numGc = Klead*[1  -Zlead]
denGc = [1 -alpha*Zlead]
Gc = tf(numGc,denGc)
disp('*******>'),pause
disp('lead controller transfer function in SS form')
Gcss = ss(Gc)
disp('*******>'),pause
disp('Series connection of lead & satellite')
Gseries = Gcss*Gss;
disp('*******>'),pause
[As, Bs, Cs, Ds] = ssdata(Gseries)
disp('*******>'),pause
disp('Open-loop eigenvalues (poles)')
pOL=pole(Gseries)
disp('*******>'),pause
disp('Open-loop zeros')
zOL=tzero(Gseries)
disp('*******>'),pause
disp('SS model of closed-loop system with only theta')
disp('.....fedback via H(s) = 1')
outputs = 1;	% feed back only theta
inputs = 1;	% input is theta_des, with neg sign on theta
Gf = feedback(Gseries,1,inputs,outputs);
[Af, Bf, Cf, Df] = ssdata(Gf)
disp('*******>'),pause
disp('Closed-loop eigenvalues')
pCL=pole(Gf)
disp('*******>'),pause
disp('Closed-loop zeros')
zCL=tzero(Gf)
disp('*******>'),pause
disp('CL step response of pointing angle theta & wheel speed')
%---- define time vector from 0 to 16 2/3 minutes
t = [0:2:1000]';
%---- initial state vector 
% states are [lead; theta; theta-dot; wheel speed in rad/s]
disp('A reasonable value is 1000 rpm')
rpm0 = input('Enter initial wheel speed in rpm ==>');
rpm_str = num2str(rpm0);
x0 = [0; 0; 0; (2*pi/60)*rpm0];
u = ones(size(t));
% delay step for 500 seconds to settle transients
for i = 1:249, u(i) = 0; end	
[y,x] = lsim(Gf,u,t,x0);
theta = y(:,1);
Omega = y(:,2);
subplot(211)
plot(t,theta,t,u,'--');grid
xlabel('Time (secs)'); ylabel('Theta (deg)')
title('CP4.2: CL satellite, pointing angle & wheel speed')
text('Units','norm','Pos',[0.05 0.8],...
       'Str',['Initial wheel speed = ' rpm_str ' rpm'])
text(700,1.20,['desired theta'])
subplot(212)
plot(t,Omega);grid
xlabel('Time (secs)'); ylabel('Omega (rpm)')
text('Units','norm','Pos',[0.1 0.4],...
       'Str',['Klead = ' Klead_str])
text('Units','norm','Pos',[0.1 0.3],...
       'Str',['Zlead = ' Zlead_str])
text('Units','norm','Pos',[0.1 0.2],...
       'Str',['alpha = ' alpha_str])
keyboard
subplot(1,1,1)
disp('end of Comprehensive Problem 4.2')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


