%%%%%%%%%%%%%%%%%%%%%% ch5ex.m %%%%%%%%%%%%%%%%%%%%%%
%       Feedback Control Problems with MATLAB
%           and the Control System Toolbox
%      D. K. Frederick and J. H. Chow, Dec. 98
%----------------------------------------------------
%  This MATLAB file runs the exercises that appear in
%  Chapter 4  (Root Locus Plots) in the BookWare 
%  Companion Feedback Control Problems Using MATLAB
%         by D. K. Frederick and J. H. Chow 
%----------------------------------------------------
clear,clf
%  menu to select Example to be run
example = -1;	% force menu to appear
while (example < 1) | (example > 3),
  disp(' 1 <= Example 5.1: Real poles only')
  disp(' 2 <= Example 5.2: Real and complex poles')
  disp(' 3 <= Example 5.3: Variable zero location')
  example = input('Select an example, or enter 0 to return to chapter menu ==> ');

if example == 0, disp('Completed Chapter 5 examples'), return, end

if example == 1,
  disp('Example 5.1: Real poles only')
  %                  K (s+5)
  %     G(s) = -------------------
  %              (s+1)(s+3)(s+12)
  %------------------------------
  disp('  ');clf
  disp('Create zero and pole arrays of G(s)')
  zOL = -5
  pOL = [ -1; -3; -12 ]
  disp('*****>');pause
  disp('Create G(s) as a ZP object')
  G = zpk(zOL,pOL,1)
  disp('*******>');pause
  disp('>>> Next, draw root-locus plot with default area')
  rlocus(G)
  %axis('equal')	% equal scaling for both axes
  grid;title('Ex 5.1: Root Locus with default area')
  keyboard
  disp('Redraw root-locus plot with smaller area ')
  axis equal
  axis([-10 10 -10 10]);
  title('Ex 5.1: Root Locus with smaller area')
  keyboard
  disp('Use mouse or arrow keys to select ')
  disp('a point on the locus that will appear.')
  disp('To repeat, enter the command')
  disp('      [kk,clroots] = rlocfind(G)')
  [kk,clroots] = rlocfind(G)
  keyboard
  clear zOL pOL G kk clroots
  disp('end of Example 5.1'); disp('  ')
  example =-1;	% force menu to appear
end	% of example 5.1 so return to menu
%-------------------------------------

if example == 2,
  disp('Example 5.2: Real and complex poles')
  %                  K (s+8)
  %     G(s) = -------------------
  %              s(s+2)(s^2+8s+32)
  disp('  ');clf
  disp('Create G(s) as TF object')
  den = conv([1 2 0],[1 8 32])  % denominator coefficients
  G = tf([1 8],den)
  disp('*******>');pause
  disp('zero, poles, and gain of G(s)')
  Gzp = zpk(G)
  disp('*******>');pause
  disp('Draw root-locus plot with specified area ')
  set(0,'DefaultLineLinewidth',1.5)
  %set(0,'DefaultMarkersize',1.5)
  rlocus(G)
  disp('*******>');pause
  axis('equal')		% same scaling for both axes
  % plot for -15 < real < 5; use dummy numbers for imag values
  axis([-15 5 -10 10]);	
  grid;title('Ex 5.2: Root Locus plotted for -15 < real < 5 & equal scaling')
  keyboard
  disp('Use mouse or arrow keys to select ')
  disp('a point on the locus that will appear.')
  disp('To repeat, enter the command')
  disp('      [kk,clroots] = rlocfind(G)')
 %% disp('*******>');pause
  [kk,clroots] = rlocfind(G)
  keyboard
  clear den G zOl pOL k kk clroots
  disp('end of Example 5.2'); disp('  ')
  example =-1;	% force menu to appear
end	% of Example 5.2 so return to menu
%-------------------------------------

if example == 3,
  disp('Example 5.3: Variable zero location')
  %              K (s+alpha)
  %     G(s) = -----------------
  %              (s+1)(s+3)(s+12)
  %%---- do locus for  K=5 & 2 <= alpha <= 10
  disp('  ');clf
  disp('Create denominator of G(s) as a polynomial & fix the gain at 5')
  temp = conv([1 1],[1 3]);
  den = conv(temp,[1 12])
  K = 5
  disp('>>> Next, construct the numerator with zero at s = alpha, where alpha ')
  disp('   takes on integer values in the interval 2 <= alpha <= 10')
  disp('*******>');pause
  clpoles = []; param = [];
  for alpha = 2.0:10,
     num = [0 0 K K*alpha];
     clpoly = num + den;  % form closed-loop char poly
     clp = roots(clpoly);  % calc closed-loop poles
     clpoles = [clpoles; clp'];
     param = [param; alpha];
  end
  disp('    alpha                         closed-loop poles ')
  disp('--------------------------------------------------------------------------')
  disp([param clpoles])
  disp('Plot the closed-loop poles with the symbol "*"')
  plot(clpoles,'*')
  axis('equal')
  grid;title('Ex 5.3: Root Locus, K = 5, 2 <= alpha <= 10')
  xlabel('Real Axis'); ylabel('Imag Axis')
  keyboard
  axis([-4 0 -2 2])	% replot area close to imag axis
  keyboard
  clear  temp den K n alpha num clpoly clp clpoles param
  disp('end of Example 5.3'); disp('  ')
  example =-1;	% force menu to appear
end	% of Example 5.3 so return to menu
%-------------------------------------
end		% of while
axis([1 2 3 4]);axis;		% resume auto scaling
axis('normal')		% restore normal aspect ratio
disp('end of Chapter 4 examples')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
