%%%%%%%%%%%%%%%%%%%%%% cp8_3a.m %%%%%%%%%%%%%%%%%%%%%
%       Feedback Control Problems with MATLAB
%           and the Control System Toolbox
%      D. K. Frederick and J. H. Chow, May 1999
%             last revision: 6/14/99 HS
%----------------------------------------------------
%	     		  Comprehensive Problem CP8.3a
% SS model of satellite (nonzero initial wheel speed)
%       Use pole with PD to avoid improper TFs
%  Does step response with u = 0 to get initial state
%   then simulates step response with PID controller
%         User enters values for KP, KI, & KD
%----------------------------------------------------
disp('CP8.3A: SS model sat (NONzero initial wheel speed),PID ctrl')
clear,clf
% load sat.mat if exists, otherwise quit this M-file
if chkmat('sat') == 1, 
    load sat.mat
  else
     return
end
%
Ap = A; Bp = B; Cp = C; Dp = D;   % plant model matrices
Gpss = ss(Ap,Bp,Cp,Dp);
%Gp = tf(Gpss);
omega0 = input('Enter initial wheel speed in rpm ==> ')
disp('Use gains from CP8.2, or try  KD = 20')
disp('Start with KI = 0, then try KI = 0.005')
KP = -0.001;	%use this value to get ICs for gain sweep
KI = input('enter integral gain KI ==> ');
KD = input('enter derivative gain KD ==> ');
KP_str=num2str(KP);
KI_str=num2str(KI);
KD_str=num2str(KD);
omega0_str = num2str(omega0);
T = 0.05*KD;	% add pole to avoid improper Gc(s)
T_str = num2str(T);
num_xx = [(KD+T) (1+(KI*T)) KI];    % ctrlr num w/o gain
numGc = KP*num_xx;   % numerator of PID
if isempty(numGc) == 1, 
   disp('Must enter numerical values for KI & KD')
   disp('....because controller numerator is empty')
   numGc
   break
end
denGc = [T 1 0];     % denominator "  " 
Gc = tf(numGc,denGc)
Gcss = ss(Gc)
[Ac,Bc,Cc,Dc] = ssdata(Gcss);   % SS controller
%--- series combination of controller & satellite
G_ol = Gpss*Gcss;
[Aol,Bol,Col,Dol] = ssdata(G_ol)
%
%------- closed-loop step response w/ zero input
%   feed back only 1st output (theta)
outputs = 1;
inputs  = 1;
G_cl = feedback(ss(G_ol),1,inputs,outputs)
[Acl,Bcl,Ccl,Dcl] = ssdata(G_cl);
t = [0:1:600]';
u = zeros(size(t));
[nstates,dummy] = size(Acl);  % number of states in CL system
x0 = [0 0 (2*pi/60)*omega0 0 0]';   % start with IC on omega
[ys,t,x] = lsim(G_cl,u,t,x0);
%----- set  x(0) = final state 
[nn,mm] = size(x);
x0 = x(nn,:);
%============== gain sweep for step in theta_desired ========
%
%    use initial state from previous run with KP = -0.001
u = ones(size(t));	% input is unit step starting at 200 sec
for i=1:max(find(t<200)), u(i)=0;end
%
for KP = [-0.0005:-0.0005:-0.0020];
   numGc = KP*num_xx;
   Gc = tf(numGc,denGc);
   Gcss = ss(Gc)
  [Ac,Bc,Cc,Dc] = ssdata(Gcss);   % SS controller
  %--- series combination of controller & satellite
  G_ol = Gpss*Gcss
  [Aol,Bol,Col,Dol] = ssdata(G_ol);
  G_cl = feedback(ss(G_ol),1,inputs,outputs)
  [Acl,Bcl,Ccl,Dcl] = ssdata(G_cl);
  ys = lsim(G_cl,u,t,x0);  % use IC for equilib.
  subplot(2,1,1)
  plot(t,ys(:,1),t,u,'--')
  xlabel('Time (s)'),ylabel('theta (deg)')
  if ishold ~=  1, hold on, end
  subplot(2,1,2)
  plot(t,ys(:,2))
  xlabel('Time (s)'),ylabel('wheel speed (rpm)')
  if ishold ~=  1, hold on, end
  disp('KP = ');disp(KP)
  disp('****>'),pause
end
hold off
subplot(211);grid
title('CP8.3a: CL satellite; ref step, PID, NONZERO wheel speed')
subplot(212);grid
text('Units','Normalized','Position',[0.5,0.55],...
	'String',['initial wheel speed = ' omega0_str ' rpm'])
text('Units','Normalized','Position',[0.4,0.40],...
	'String','KP = -0.0005, -0.0010, -0.0015, -0.0020')
text('Units','Normalized','Position',[0.5,0.25],...
	'String',['KI = ' KI_str])
text('Units','Normalized','Position',[0.5,0.1],...
	'String',['KD = ' KD_str])
%
keyboard
disp('end of Comprehensive Problem 8.3a')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
