%%%%%%%%%%%%%%%%%%%%%% ex8_1.m %%%%%%%%%%%%%%%%%%%%%%
%       Feedback Control Problems with MATLAB
%           and the Control System Toolbox
%      D. K. Frederick and J. H. Chow, Jan. 99
%----------------------------------------------------
%                  P-only control
%----------------------------------------------------
clear,clf
disp('Example 8.1: Proportional control')
%----- plant & sensor parameters
tauP1 = 2; tauP2 = 0.5; tau_sen = 0.05; Kproc = 4;
denGp = conv([tauP1 1],[tauP2 1]);
Gp    = tf(Kproc,denGp)
H     = tf(1,[tau_sen 1])
%---------- open-loop transfer function
GpH   = Gp*H;
%=============== Part (a): root locus =============
disp('Part (a): root locus for variations in Kp')
rlocus(GpH);grid
%axis equal
title('Ex 8.1: open-loop, proportional control')
disp('first, drawn with default axes')
keyboard
disp('redraw with better axes')
title('Ex 8.1: open-loop, proportional control, Fig 8.2(a)')
sgrid(0.8,[])
axis equal
axis([-25 5 -15 15])
text(-14,11,'zeta = 0.8')
keyboard
disp('Find gain and closed-loop poles on jw-axis')
[kk,polesCL] = rlocfind(GpH)
keyboard
disp('redraw again with better resolution near orign')
axis([-4 1 -2 2])
disp('Find gain and closed-loop poles with zeta = 0.8')
[kk,polesCL] = rlocfind(GpH)
keyboard
disp('Find gain and closed-loop poles at breakaway point')
[kk,polesCL] = rlocfind(GpH)
keyboard
%============ Part (b) gain sweep =================
disp('Part (b): CL step responses with gain sweep')
disp(' ')
disp('Values in Table 8.1')
disp(' ')
%---- closed-loop step response to reference input
t = [0:0.02:10];
disp('    KP    ymax      t_p        y_ss     Mo') 
disp(' ')
for KP = [0.7:0.2:1.7];
  T = feedback(KP*Gp,H,-1);
  ys = step(T,t);
  ref = dcgain(T);
  [Mo,tp,tr,ts,ess] = tstats(t,ys,ref);
  ymax = ys(find(t==tp));
  yss = ys(length(ys));
  disp([KP ymax tp yss  Mo])
  disp('*****>');pause
  plot(t,ys)
  if ishold ~=  1, hold on, end
end
xlabel('Time (sec.)'),ylabel('Amplitude')
grid;title('Ex 8.1: CL step resp, Prop control, Fig8.2(b)')
text(5.0,0.25,'KP = 0.7,0.9, ... ,1.7')
text(6.0,0.70,'KP = 0.7')
text(6.0,0.92,'KP = 1.7')
hold off
axis([0 10 0 1.2])
keyboard
%================= Part (c): CL step responses ==========
disp('Part (c): CL step responses for KP = 1.255')
KP = 1.255
KP_str=num2str(KP);
%----- closed-loop step response to reference input
T_ref = feedback(KP*Gp,H,-1)
t = [0:0.02:10];
y_ref = step(T_ref,t);

%=========== repeat for disturbance input ===========
T_dist = feedback(Gp,-KP*H,+1)
y_dist = step(T_dist,t);
%-------- plot both responses
plot(t,y_ref,t,y_dist,'--')
xlabel('Time (sec.)'),ylabel('Amplitude')
grid;
title('Ex 8.1: CL step responses, Prop control, Fig. 8.2(c)')
text(4.0,0.25,['KP = ' KP_str])
text(6.5,0.88,'reference')
text(6.5,0.71,'disturbance')
axis([0 10 0 1.2])
keyboard
%------- calc final values
disp('steady-state responses are:')
y_ref_ss = y_ref(length(t))
y_dist_ss = y_dist(length(t))
disp('end of Example Problem 8.1')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%