%%%%%%%%%%%%%%%%%%%%%% cp9_2.m %%%%%%%%%%%%%%%%%%%%%%
%       Feedback Control Problems with MATLAB
%           and the Control System Toolbox
%      D. K. Frederick and J. H. Chow, May 1999
%             last revision: 6/15/99 HS
%----------------------------------------------------
%    satellite TF model for motor torque --> theta
%	  lead control via Kuo's algorithm
%           code is similar to ex9_3a.m
% 
%         User enters desired phase margin
%           Computes Klead and omega_c' 
%   (freq at which uncomp system has desired phase 
%         margin, with allowance for extra)
%----------------------------------------------------
disp('CP9.2: satellite with lead control via Kuo''s algorithm')
clear;clf
% load sat.mat if exists, otherwise quit this M-file
if chkmat('sat') == 1, 
    load sat.mat
  else
     return
end
disp('satellite TF model for motor torque --> theta')
disp('Note: plant is type-1 so have zero ss error for step')
disp('Steps are:')
disp(' 1. Specify a gain like -0.001 to avoid large wheel speeds ')
disp(' 2. Add lead to get enough phase margin ')
disp('')
%----- rename TF vectors read from file
numGp = numG1;
denGp = denG1;
Gp = tf(numGp,denGp);
%-------- will have zero steady-state error with lead, so 
%	just set a value. Would like as high as possible
%	if want to use with nonzero initial wheel speed
%	or should use lead-lag for that situation.
disp('Can start with Klead = -0.001')
Klead = input('enter controller gain (< 0) ==> ')
Klead_str = num2str(Klead);
%---------- use open-loop transfer function to calc gain
%		req'd to provide desired s-s error
%    first, cancel pole & zero at origin
KGp  = minreal(Klead*Gp);
disp('*********>');pause
%===== Part(a) determine phase margin for gain-only
%
[km,pm,wkm,wpm] = margin(KGp) % pm with gain only
disp('*********>');pause
%====== Part (b) determine lead alpha
w = logspace(-1,2);
[mag,ph] = bode(KGp,w);
disp('reasonable value for  phase-margin target is 70 deg')
pm_target = input('phase-margin target in degrees ==> ')
del_pm = pm_target - pm        % max phase angle in deg 
Alead = (1+sin(del_pm*pi/180))/(1-sin(del_pm*pi/180)) % Eq (9.2)
Alead_str = num2str(Alead);
disp('*********>');pause
%====== Part (c) determine lead zero
wc = interp1(mag(:),w,1/sqrt(Alead)) % use (9.4) to find wc 
Zlead = -wc/sqrt(Alead)      % using (9.3)
Zlead_str = num2str(Zlead);
disp('*********>');pause
numGc = Klead*Alead*[1  -Zlead];   % use (9.1) for lead TF
denGc = [1  -Alead*Zlead];
Gc = tf(numGc,denGc);
%---- open-loop transfer function of compensated system
GpGc = Gp*Gc;
%---- get phase margin of final design
GpGc = minreal(GpGc);  % Note to users: minreal is a function 
% to provide a minimal realization of the transfer function. 
% In this system, there is an integrator that is not changed 
% by the control. For a proper analysis, the pole at the 
% origin associated with the integrator should be removed 
% first. 
[km,pm,wkm,wpm] = margin(GpGc)
%
%==========  OL frequency responses for Fig. 9.8 =====
w = logspace(-3,2);
ph180 = -180*ones(length(w));
%---- first, with gain only
[mag_K,ph_K] = bode(Klead*Gp,w);
mag_db_K = 20*log10(mag_K);
%-------- then with final lead design
[mag_lead,ph_lead] = bode(GpGc,w);
mag_db_lead = 20*log10(mag_lead);
%--------- now plot both sets
subplot(2,1,1)
semilogx(w,mag_db_lead(:),'-',w,mag_db_K(:),'--');grid;
xlabel('Frequency (rad/s)')
ylabel('Magnitude (dB)')
title('CP9.2: OL freq response ')
text(0.005,-60,['Klead = ' Klead_str])
text(0.005,-100,['Zlead = ' Zlead_str])
text(0.005,-140,['Alead = ' Alead_str])
subplot(2,1,2)
semilogx(w,ph_lead(:),'-',w,ph_K(:),'--',w,ph180,'-');grid
xlabel('Frequency (rad/s)')
ylabel('Phase angle (deg)')
keyboard

%==========  CL step response  =====
disp('CL step response')
%---  reference input: fwd path = Gp*Gc;  fdbk path = 1
GpGc_CL = feedback(GpGc,1);
t = [0:1.0:200];
ys = step(GpGc_CL,t);
%-------- plot response
subplot(111)
plot(t,ys);grid
xlabel('Time (s)'),ylabel('output')
title('CP9.2: Satellite with lead ctrlr; CL step response')
text(100,0.60,['Klead = ' Klead_str])
text(100,0.50,['Zlead = ' Zlead_str])
text(100,0.40,['Alead = ' Alead_str])
disp('end of Comprehensive Problem 9.2')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%