%%%%%%%%%%%%%%%%%%%%%% ex9_2.m %%%%%%%%%%%%%%%%%%%%%%
%       Feedback Control Problems with MATLAB
%           and the Control System Toolbox
%      D. K. Frederick and J. H. Chow, Jan. 99
%----------------------------------------------------
clear,clf
disp('Lag control via Kuo''s algorithm')
%----- plant & sensor parameters
tauP1 = 2; tauP2 = 0.5; tau_sen = 0.05; Kproc = 4; 
Gp = tf(Kproc,conv([tauP1 1],[tauP2 1]));
H  = tf(1,[tau_sen 1]);
%---------- open-loop transfer function
GpH = Gp*H;
%-- position constant req'd for 2% steady-state error
Kp_des = (1/0.02) - 1
%----use open-loop transfer function to calc gain
%		req'd to provide desired s-s error
plant_lfg = dcgain(GpH);
Klag = Kp_des/plant_lfg
Klag_str=num2str(Klag);	
%-------------- open-loop frequency response
w = logspace(-1,1,100)';
zdb = 0*w;
ph180 = -180*ones(length(w));
%------ gain-only
[mag,ph] = bode(Klag*GpH,w);
mag   = reshape(mag,100,1);
ph    = reshape(ph, 100,1);
mag_db = 20*log10(mag);
subplot(211)
semilogx(w,mag_db,w,zdb);grid
title('Ex 9.2: plant + gain to give desired ss error')
ylabel('magnitude - dB')
text(0.2,5,['Klag = ' Klag_str])
text(0.2,-5,'controller is gain only')
subplot(212)
semilogx(w,ph,w,ph180);grid
xlabel('frequency - rad/sec')
subplot(111)
[km,pm,wkm,wpm] = margin(Klag*GpH);
disp('gain margin in dB & omega_km are:')
disp([20*log10(km)  wkm])
disp('phase margin in degrees & omega_pm are:')
disp([pm wpm])
keyboard
disp('Now determine desired magnitude')
disp('               cross-over freq for final system')
wc = interp1(ph,w,-115)
wc_str=num2str(wc);
disp('*********>');pause
%==========================
%---- do freq response at only one freq
[mag_KGpH,ph_KGpH] = bode(Klag*GpH,wc)
Alag = mag_KGpH
Alag_str = num2str(Alag);
disp('using lag zero corner freq = wc/10')
Zlag = -wc/10
Zlag_str = num2str(Zlag);
disp('*********>');pause

%---- using Kuo's form for lag (unity low-freq gain)
%	rather than (3.2) in book
Gc = tf(Klag*[-1/Zlag  1],[-Alag/Zlag  1]);	 
%---- open-loop transfer function of compensated system
GcGpH = Gc*GpH;
%------- compute low-freq gain of current design
disp('low-freq gain of current design is:')
lfg = dcgain(GcGpH)
%
%-------------- open-loop frequency response
[km,pm,wkm,wpm] = margin(GcGpH);
disp('gain margin in dB & omega_km are:')
disp([20*log10(km)  wkm])
disp('phase margin in degrees & omega_pm are:')
disp([pm wpm])
disp('*********>');pause
%---- do the plot
margin(GcGpH)
text(0.01,-160,['Klag = ' Klag_str])
text(0.01,-200,['Zlag = ' Zlag_str])
text(0.01,-240,['Alag = ' Alag_str])
subplot(2,1,1)
text(0.01,-50,['Using crossover freq =  ' wc_str ' rad/s'])
%
%==========  OL frequency responses for Fig. 9.6 =====
w = logspace(-3,2,100);
ph180 = -180*ones(length(w));
%---- first, with gain only
[mag_K,ph_K] = bode(Klag*GpH,w);
mag_K = reshape(mag_K,100,1);
ph_K  = reshape(ph_K ,100,1);
mag_db_K = 20*log10(mag_K);
%-------- then with final lag design
[mag_lag,ph_lag] = bode(GcGpH,w);
mag_lag = reshape(mag_lag,100,1);
ph_lag  = reshape(ph_lag ,100,1);
mag_db_lag = 20*log10(mag_lag);
%--------- now plot both sets
subplot(211)
semilogx(w,mag_db_lag,'-',w,mag_db_K,'--');grid;
xlabel('Frequency (rad/s)')
ylabel('Magnitude (dB)')
title('Ex 9.2: OL freq response per Fig. 9.6')
subplot(212)
semilogx(w,ph_lag,'-',w,ph_K,'--',w,ph180,'-');grid
xlabel('Frequency (rad/s)')
ylabel('Phase angle (deg)')
keyboard
subplot(111)	% reset plot window

%==========  CL step responses for Fig. 9.7 =====
%---  reference input: fwd path = Gc*Gp;  fdbk path = H
T_ref = feedback(Gp*Gc,H,-1)
t = [0:0.02:10];
y_ref = step(T_ref,t);
%=========== repeat for disturbance input ===========
%--- dist input: fwd path = Gp; fdbk path = -H*Gc
%------ closed-loop step response to disturbance input
%--- use +1 for feedback gain because neg sign is in H(s)
T_dist = feedback(Gp,-Gc*H,+1)
y_dist = step(T_dist,t);
%-------- plot both responses
plot(t,y_ref,t,y_dist,'--')
xlabel('Time (sec.)'),ylabel('output')
grid;
title('Ex 9.2: CL ref & dist step responses Fig. 9.7')
text(2.0,0.25,['Klag = ' Klag_str])
text(2.0,0.15,['Zlag = ' Zlag_str])
text(2.0,0.05,['Alag = ' Alag_str])
axis([0 10 0 1.2])
disp('end of Example Problem 9.2')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%